==> atajo-ionic-0.0.15.tgz: package/src/lib/api/index.ts <==
import { Injectable } from '@angular/core';
import { Config } from '../config';
import { Observable } from "rxjs/Observable";
import { Payload } from '../payload';
import { Events } from 'ionic-angular';

import { Socket } from '../socket';
import { Database } from '../database';
import { Status } from '../status';

@Injectable()
export class Api {


    private static transactions: object = null;
    private static timeoutCheckInterval: any;

    constructor(private appConfig: Config, private socketService: Socket, private payloadProvider: Payload, private events: Events) {

        if (!Api.transactions) { Api.transactions = {}; }
        this.events.subscribe('socket:client:rx', this.transactionResponse);

    }

    public addTransaction(lambda: string, request: object, opts: object = {}) {

        return Observable.create(observer => {

            this.payloadProvider.getPayload(request).then(payload => {


                let timeout = opts['timeout'] || this.appConfig.get('api.timeout', 10000);
                let queue = opts['queue'] || false;
                let pid = payload['pid'];

                if (!queue && !this.socketService.connected()) {

                    observer.error({ progress: 0, status: { code: Status.NOT_CONNECTED, message: Status.get(Status.NOT_CONNECTED) }, data: {} });
                    return;
                }


                console.log("ADD TRANSACTION FOR " + lambda + ": " + pid);


                Api.transactions[pid] = {

                    timeout: timeout,
                    latency: {
                        deviceSubmitAt: new Date().getTime(),
                    },
                    payload: payload,
                    at: new Date().getTime(),
                    status: 'SUBMIT',
                    observer: observer,
                    lambda: lambda,
                    queue: queue,
                    version: this.appConfig.get('core', false).version
                }

                Api.startTimeoutCheck();
                this.processTransaction(pid);

            });

        });

    }


    private transactionResponse(rx) {

        if (rx.pid) {


            console.log("API:CLIENT_RX : ", rx);
            let pid = rx.pid;
            let transaction = Api.transactions[pid];
            if (transaction) {
                let observer = transaction.observer;

                //CALC PERCENTAGES FROM CHUNKS
                let chunkId = rx.response.chunkId + 1;
                let chunkTotal = rx.response.chunkTotal;
                let progress = (100 / chunkTotal) * chunkId;

                if (!rx.response.data) rx.response.data = [];

                if (Array.isArray(rx.response.data)) {
                    if (rx.response.data.length == 1) {
                        rx.response.data = rx.response.data[0];
                    }
                }

                if (rx.error) {
                    observer.error({ progress: progress, data: rx.response.data });
                }
                else {
                    observer.next({ progress: progress, data: rx.response.data });
                }

                observer.complete();
            }
            else {
                console.warn("TRANSACTION NOT FOUND FOR " + pid, rx);
            }


        }
        else return console.error("SOCKET:TX_RESPONSE INVALID : ", rx);



    }


    private processTransaction(pid: string) {

        console.log("PROCESS TRANSACTION : " + pid);
        let that = this;

        let transaction = Api.transactions[pid];
        let observer = transaction.observer;
        let queue = transaction.queue;

        if (queue) {
            //TODO : IMPLEMENT QUEUING
        }
        else {

            //MAKE THE CALL; 
            transaction.latency.deviceRequestAt = new Date().getTime();
            this.socketService.transaction(transaction);



        }

    }



    private static checkTimeouts() {

        if (Object.keys(Api.transactions).length == 0) {
            Api.stopTimeoutCheck();
        }


        console.debug("CHECKING TIMEOUTS");

        for (var pid in Api.transactions) {
            let tx = Api.transactions[pid];
            let now = new Date().getTime();
            console.debug("IS " + now + " - " + tx.at + " > " + tx.timeout);
            if ((now - tx.at) > tx.timeout) {
                console.debug("TX FOR PID " + pid + " TIMED OUT");
                tx.observer.error({ progress: 0, data: "Transaction Timed Out" });
                delete Api.transactions[pid];
            }
        }

    }

    private static startTimeoutCheck() {

        if (!Api.timeoutCheckInterval) { Api.timeoutCheckInterval = setInterval(Api.checkTimeouts, 1000); }

    }

    private static stopTimeoutCheck() {

        if (Api.timeoutCheckInterval) clearInterval(Api.timeoutCheckInterval); Api.timeoutCheckInterval = false;

    }



}

==> atajo-ionic-0.0.23.tgz: package/src/lib/api/index.ts <==
import { Injectable } from '@angular/core';
import { Config } from '../config';
import { Observable } from "rxjs/Observable";
import { Payload } from '../payload';
import { Events } from 'ionic-angular';

import { Socket } from '../socket';
import { Database } from '../database';
import { Status } from '../status';

@Injectable()
export class Api {


    private static transactions: object = null;
    private static timeoutCheckInterval: any;

    constructor(private appConfig: Config, private socketService: Socket, private payloadProvider: Payload, private events: Events, private status: Status) {

        if (!Api.transactions) { Api.transactions = {}; }
        this.events.subscribe('socket:client:rx', this.transactionResponse);

    }

    public addTransaction(lambda: string, request: object, opts: object = {}) {

        return Observable.create(observer => {

            this.payloadProvider.getPayload(request).then(payload => {


                let timeout = opts['timeout'] || this.appConfig.get('api.timeout', 10000);
                let queue = opts['queue'] || false;
                let pid = payload['pid'];
                if (!queue && !this.socketService.connected()) {

                    console.log("SOCKET IS NOT CONNECTED");

                    observer.error({ progress: 0, status: { code: Status.NOT_CONNECTED, message: Status.get(Status.NOT_CONNECTED) }, data: {} });
                    return;
                }


                console.log("ADD TRANSACTION FOR " + lambda + ": " + pid);


                Api.transactions[pid] = {

                    timeout: timeout,
                    latency: {
                        deviceSubmitAt: new Date().getTime(),
                    },
                    payload: payload,
                    at: new Date().getTime(),
                    status: 'SUBMIT',
                    observer: observer,
                    lambda: lambda,
                    queue: queue,
                    version: this.appConfig.get('core', false).version
                }

                Api.startTimeoutCheck();
                this.processTransaction(pid);

            });

        });

    }


    private transactionResponse(rx) {

        if (rx.pid) {


            console.log("API:CLIENT_RX : ", rx);
            let pid = rx.pid;
            let transaction = Api.transactions[pid];
            if (transaction) {
                let observer = transaction.observer;

                //CALC PERCENTAGES FROM CHUNKS
                let chunkId = rx.response.chunkId + 1;
                let chunkTotal = rx.response.chunkTotal;
                let progress = (100 / chunkTotal) * chunkId;

                if (!rx.response.data) rx.response.data = [];

                if (Array.isArray(rx.response.data)) {
                    if (rx.response.data.length == 1) {
                        rx.response.data = rx.response.data[0];
                    }
                }

                if (rx.error) {
                    observer.error({ progress: progress, data: rx.response.data });
                }
                else {
                    observer.next({ progress: progress, data: rx.response.data });
                }

                delete Api.transactions[pid];
                observer.complete();
            }
            else {
                console.warn("TRANSACTION HAS BEEN REMOVED (TIMED OUT) FOR " + pid, rx);
            }


        }
        else return console.error("SOCKET:TX_RESPONSE INVALID : ", rx);



    }


    private processTransaction(pid: string) {

        console.log("PROCESS TRANSACTION : " + pid);
        let that = this;

        let transaction = Api.transactions[pid];
        let observer = transaction.observer;
        let queue = transaction.queue;

        if (queue) {
            //TODO : IMPLEMENT QUEUING
        }
        else {

            //MAKE THE CALL; 
            transaction.latency.deviceRequestAt = new Date().getTime();
            this.socketService.transaction(transaction);



        }

    }



    private static checkTimeouts() {

        if (Object.keys(Api.transactions).length == 0) {
            Api.stopTimeoutCheck();
            return;
        }


        // console.debug("CHECKING TIMEOUTS");

        for (var pid in Api.transactions) {
            let tx = Api.transactions[pid];
            let now = new Date().getTime();
            // console.debug("IS " + now + " - " + tx.at + " > " + tx.timeout);
            if ((now - tx.at) > tx.timeout) {
                console.debug("TX FOR PID " + pid + " TIMED OUT");
                tx.observer.error({ progress: 0, data: "Transaction Timed Out" });
                delete Api.transactions[pid];
            }
        }

    }

    private static startTimeoutCheck() {

        if (!Api.timeoutCheckInterval) { Api.timeoutCheckInterval = setInterval(Api.checkTimeouts, 1000); }

    }

    private static stopTimeoutCheck() {

        if (Api.timeoutCheckInterval) clearInterval(Api.timeoutCheckInterval); Api.timeoutCheckInterval = false;

    }



}
